#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "misc.h"
#include "Book.h"
#include "BookActionServer.h"
#include "list.h"
#include "Messages.h"
#include "intersection.h"
#include "Table.h"

Table BookTable=NULL;
void InitBookServer(void)
     {
	BookTable=AllocateTable(BOOK_FIELD_COUNT);
     }

struct aantaldubbelscommon  
     {
	Book t;
	int aantal;
     };

static void* aantaldubbels(Book b, struct aantaldubbelscommon *c)
     {
	assert(b && c);
	if (VoldoetRecordAanTemplate((Record)c->t,(Record)b,BOOK_FIELD_COUNT)) 
	  {
	     int bezet;
	     bezet=atoi(b->Versie);
	     if (bezet>=c->aantal) c->aantal=bezet+1;
	  }
	return (void*)b;
     }

void ADDBOOK(char*Message)
     {
	Book m;
	struct aantaldubbelscommon t;
	char a=ADDBOOK_RP_OK,exempl[30];
	m=RecvRecord(BOOK_FIELD_COUNT);
	/* zoek eerst dubbeltjes op... */
	t.t=CreateCompareableRecord(m,BOOK_FIELD_COUNT);
	t.aantal=0;
	TableForeach(BookTable,aantaldubbels,&t);
	/* ken exemplaarnummertoe */
	zfree(m->Versie);
	sprintf(exempl,"%d",t.aantal);
	m->Versie=strdup(exempl);
	/* toevoegen **/
	AddRecord(BookTable,(Record)m);
	/* antwoord sturen */
	FreeRecord(t.t,BOOK_FIELD_COUNT);
	SendChar(a);
	SendInteger(t.aantal);
	FlushMessage();
	printf("BookServer: Added book '%s(%s)'\n",m->Name,m->Versie);
     }

void DELETEBOOK(char*Message)
     {
	Book m=(Book)RecvRecord(BOOK_FIELD_COUNT);
	DeleteRecord(BookTable,m);
	SendChar(DELETEBOOK_RP_OK);
	printf("BookServer: Deleted book '%s: %s'\n",m->Rubriek,m->Name);
	FlushMessage();
	FreeRecord(m,BOOK_FIELD_COUNT);
     }

void UPDATEBOOK(char*Message)
     {
	Book o,n;
	char a;
	o=(Book)RecvRecord(BOOK_FIELD_COUNT);
	n=(Book)RecvRecord(BOOK_FIELD_COUNT);
	a=UpdateRecord(BookTable,(Record)o,(Record)n) ? UPDATEBOOK_RP_OK : UPDATEBOOK_RP_NOK;
	SendChar(a);
	printf("BookServer: %spdated book '%s'\n",a==UPDATEBOOK_RP_OK ? "U" : "Not u",o->Name);
	FlushMessage();
	FreeRecord(o,BOOK_FIELD_COUNT);
	FreeRecord(n,BOOK_FIELD_COUNT);
     }

void FINDBOOKS(char*Message)
     {
	Book b;
	int aantal;
	char a=FINDBOOKS_RP_OK;
	b=(Book)RecvRecord(BOOK_FIELD_COUNT);
	SendChar(FINDBOOKS_RP_OK);
	SendChunck(&aantal,4,dontfree);
	aantal=htonl(FindRecordsAndAnswer(BookTable,b));
	FreeRecord(b,BOOK_FIELD_COUNT);
	FlushMessage();
     }

void* SaveSingleBook(void* who, void* common)
     {
	FILE * F=(FILE*)common;
	int size;
	char*r;
	Book m=(Book)who;
	assert(m);
	assert(F);
	size=RecordFormattedSize(m,BOOK_FIELD_COUNT);
	r=(char*)zmalloc(size);
	FormatRecord(r,(Record)m,BOOK_FIELD_COUNT);
	if ((1!=fwrite(&size,sizeof(size),1,F)) || 
	    (size!=fwrite(r,1,size,F)))
	  {
	     printf("BookServer: Couldn't write book outputfile\n");
	     exit(1);
	  };
	free(r);
	return who;
     }

int SaveBooks(const char * name)
     {
	FILE* o;
	o=fopen(name,"wb");
	if (!o) return 0;
	TableForeach(BookTable,SaveSingleBook,o);
	fclose(o);
	printf("BookServer: Saved books\n");
	return 1;
     }

int LoadBooks(const char*name)
     {
	int count;
	Book b;
	char*r;
	FILE*i;
	i=fopen(name,"rb");
	if(!i)  return 0;
	count=0;
	while(!feof(i))
	  {
	     int s;
	     if (fread(&s,sizeof(int),1,i)!=1) break;
	     r=zmalloc(s);
	     if (fread(r,s,1,i)!=1) return 0;
	     RawAddRecord(BookTable,b=(Book)UnFormatRecord(r,BOOK_FIELD_COUNT));
	       {
		  int nr=atoi(b->Key);
		  if (nr>BookTable->lastkeynumber) BookTable->lastkeynumber=nr;
	       }
	     free(r);
	     count++;
	  }
	fclose(i);
	printf("BookServer: Loaded %d books\n",count);
	return 1;
     }
