#include "ClientConnect.h"
#include "Record.h"
#include "CursussenActionClient.h"
#include "Messages.h"
#include "myio.h"
#include "misc.h"
#include "intersection.h"

int ADDCURSUS(Cursus m)
     {
	assert(m);
	SendChar(ADDCURSUS_RQ);
	SendRecord((Record)m,CURSUS_FIELD_COUNT);
	FlushMessage();
	if (RecvChar()!=ADDCURSUS_RP_OK) assert(0);
	return RecvInteger();
     }

int DELETECURSUS(Cursus m)
     {
	assert(m);
	SendChar(DELETECURSUS_RQ);
	SendRecord((Record)m,CURSUS_FIELD_COUNT);
	FlushMessage();
	return RecvChar();
     }

int UPDATECURSUS(Cursus b, Cursus n)
     {
	assert(b && n);
	SendChar(UPDATECURSUS_RQ);
	SendRecord((Record)b,CURSUS_FIELD_COUNT);
	SendRecord((Record)n,CURSUS_FIELD_COUNT);
	FlushMessage();
	return RecvChar();
     }

CursusSearchResult FINDCURSUSSEN(Cursus b)
{
   int aantal;
   CursusSearchResult result;
   assert(b);
   SendChar(FINDCURSUSSEN_RQ);
   SendRecord((Record)b,CURSUS_FIELD_COUNT);
   FlushMessage();
   if (RecvChar()!=FINDCURSUSSEN_RP_OK) 
     {
	result.result=NULL;
	result.result=0;
	return result;
     }
   /* dan nu eens converteren naar een waardevol lijstje */
   aantal=RecvInteger();
   assert(aantal>=0);
   if (!aantal) result.result=NULL;
   else result.result=(Cursus*)zmalloc(sizeof(Cursus)*aantal);
   result.aantal=aantal;
   while(aantal>0)
     {
	aantal--;
	result.result[aantal]=(Cursus)RecvRecord(CURSUS_FIELD_COUNT);
     }
   return result;
}

typedef struct sCursusSubdialog {
   inputandstatic codes,names,pagecounts,prijsds,prijsls, prijsnls, bijbestelds, comments;
} CursusSubdialog;

CursusSubdialog CreateCursusSubdialog(Cursus b)
     {
	CursusSubdialog bs;
	bs.codes=MakeInputAndStatic(2,2,10,"Code ",&b->Code);
	bs.names=MakeInputAndStatic(2,3,50,"Name ",&b->Name);
	bs.pagecounts=MakeInputAndStatic(2,4,10,"Page count ",&b->PageCount);
	bs.prijsds=MakeInputAndStatic(2,5,10,"Prijs debuggers ",&b->PrijsD);
	bs.prijsls=MakeInputAndStatic(42,5,10,"Prijs Leden ",&b->PrijsL);
	bs.prijsnls=MakeInputAndStatic(2,6,10,"Prijs niet leden ",&b->PrijsNL);
	bs.bijbestelds=MakeInputAndStatic(42,6,10,"Aantal bijbesteld ",&b->Bijbesteld);
	bs.comments=MakeInputAndStatic(2,7,60,"Comment ",&b->Comments);
	return bs;
     }

static void CurrentChanged(Cursus r,CursusSubdialog*bs)
     {
	assert(bs && r);
	PasteInputString(bs->codes.i,r->Code);
	PasteInputString(bs->names.i,r->Name);
	PasteInputString(bs->pagecounts.i,r->PageCount);
	PasteInputString(bs->bijbestelds.i,r->Bijbesteld);
	PasteInputString(bs->prijsls.i,r->PrijsL);
	PasteInputString(bs->prijsds.i,r->PrijsD);
	PasteInputString(bs->prijsnls.i,r->PrijsNL);
	PasteInputString(bs->comments.i,r->Comments);
     }

void Key(int c, Cursus r,dialog d)
     {
	if (c=='e' || c=='E')
	  {
	     EditCursus(r);
	     RedrawDialog(d);
	  }
	if (c=='d' || c=='D')
	  {
	     DeleteCursus(r);
 	     RedrawDialog(d);
	  }
     }

Cursus SelectCursusMenu(char* Title)
     {
	int answer;
	Cursus result=NULL;
	Cursus m;
	Cursus Cursusheader;
	CursusSubdialog bs;
	listbox lb;
	dialog d;
	Cursusheader=(Cursus)AllocateRecord(CURSUS_FIELD_COUNT);
	Cursusheader->Key="";
	Cursusheader->Code="  Code  ";
	Cursusheader->Name="                     Name                      ";
	Cursusheader->PageCount="  blz  ";
	Cursusheader->PrijsD="  prijs D  ";
	Cursusheader->PrijsNL="  prijs NL  ";
	Cursusheader->PrijsL="  prijs L  ";
	Cursusheader->Bijbesteld=" Bijbesteld ";
	Cursusheader->Comments="      Comments    ";
	m=(Cursus)AllocateRecord(CURSUS_FIELD_COUNT);
	bs=CreateCursusSubdialog(m);
	lb=MakeListBox(2,9,76,15,NULL,9,0,Cursusheader);
	SetListBoxCurrentChangedCallback(lb,CurrentChanged,&bs);
	d=MakeDialog(17,(!Title ? "Search cursussen" : Title),
		     bs.codes.s,bs.codes.i,
		     bs.names.s,bs.names.i,
		     bs.pagecounts.s,bs.pagecounts.i,
		     bs.prijsds.s,bs.prijsds.i,
		     bs.prijsls.s,bs.prijsls.i,
		     bs.prijsnls.s,bs.prijsnls.i,
		     bs.bijbestelds.s,bs.bijbestelds.i,
		     bs.comments.s,bs.comments.i,lb);
	if (!Title) SetListBoxKeyCallback(lb,Key,d);
	while(1)
	  {
	     if ((answer=RunDialog(d))==DIALOG_CANCEL) break;
	       {
		  CursusSearchResult sr=FINDCURSUSSEN(m);
		  SetListBoxContents(lb,sr.result,sr.aantal);
		  if (sr.aantal==0)
		    StatusLine("Geen cursussen gevonden");
		  else
		    {
		       SetDialogCurrent(d,16); /* direct naar listbox na selectie
						* op verzoek van Sven */
		       if (Title && sr.aantal==1)
			 {
			    result=sr.result[0];
			    break;
			 }
		       else result=NULL;
		    }
	       }
	  }
	FreeRecord((Record)m,CURSUS_FIELD_COUNT);
	FreeDialogAndContents(d);
	return result;
     }


/* cursus wijzigt als de server het cursus gewijzigt heeft */
void EditCursus(Cursus b)
     {
	Cursus n;
	CursusSubdialog bs;
	dialog d;
	n=(Cursus)AllocateRecord(CURSUS_FIELD_COUNT);
	PasteRecord((Record)n,(Record)b,CURSUS_FIELD_COUNT);
	bs=CreateCursusSubdialog(n);
	while (1)
	  {
	     d=MakeDialog(16,"Edit cursus",
			  bs.codes.s,bs.codes.i,
			  bs.names.s,bs.names.i,
			  bs.pagecounts.s,bs.pagecounts.i,
			  bs.prijsds.s,bs.prijsds.i,
			  bs.prijsls.s,bs.prijsls.i,
			  bs.prijsnls.s,bs.prijsnls.i,
			  bs.bijbestelds.s,bs.bijbestelds.i,
			  bs.comments.s,bs.comments.i);
	     if (RunDialog(d)==DIALOG_CANCEL) break;
	     if (UPDATECURSUS(b,n)!=UPDATECURSUS_RP_OK) StatusLine("Failed to update cursus");
	     else 
	       {
		  PasteRecord((Record)b,(Record)n,CURSUS_FIELD_COUNT);
		  StatusLine("Cursus updated successfully");
		  break;
	       }
	  }
     	FreeRecord((Record)n,CURSUS_FIELD_COUNT);
	FreeDialogAndContents(d);
     }

void DeleteCursus(Cursus b)
     {
	keuze k;
	statictext names,codes,prijsds,prijsls,prijsnls,bijbestelds,pagecounts,comments;
	dialog d;
	codes=MakeStatic(2,2,b->Code);
	names=MakeStatic(2,3,b->Name);
	pagecounts=MakeStatic(2,4,b->PageCount);
	prijsds=MakeStatic(2,5,b->PrijsD);
	prijsls=MakeStatic(42,5,b->PrijsL);
	prijsnls=MakeStatic(42,5,b->PrijsNL);
	bijbestelds=MakeStatic(42,5,b->Bijbesteld);
	comments=MakeStatic(42,5,b->Comments);
	k=MakeKeuze(1,35,9,2,"Delete curus","no, just kidding");
	d=MakeDialog(8,"Delete cursus ?",codes,names,pagecounts,prijsds,prijsls,prijsnls,bijbestelds,k);
	if (RunDialog(d)==DIALOG_OK && GetKeuzeCurrent(k)==0)
	  {
	     
	     if (DELETECURSUS(b)!=DELETECURSUS_RP_OK) StatusLine("Failed to delete cursus");
	     else StatusLine("Cursus deleted");
	  }
	FreeDialogAndContents(d);
     }

void AddCursusMenu(void)
     {
	int answer;
	Cursus m;
	CursusSubdialog bs;
	dialog d;
	while(1)
	  {
	     m=(Cursus)AllocateRecord(CURSUS_FIELD_COUNT);
	     bs=CreateCursusSubdialog(m);
	     d=MakeDialog(16,"Add cursus",
			  bs.codes.s,bs.codes.i,
			  bs.names.s,bs.names.i,
			  bs.pagecounts.s,bs.pagecounts.i,
			  bs.prijsds.s,bs.prijsds.i,
			  bs.prijsls.s,bs.prijsls.i,
			  bs.prijsnls.s,bs.prijsnls.i,
			  bs.bijbestelds.s,bs.bijbestelds.i,
			  bs.comments.s,bs.comments.i
			  );
	     if ((answer=RunDialog(d))==DIALOG_CANCEL) break;
	     ADDCURSUS(m);
	     StatusLine("Cursus added");
	     FreeRecord((Record)m,CURSUS_FIELD_COUNT);
	  }
	FreeRecord((Record)m,CURSUS_FIELD_COUNT);
	FreeDialogAndContents(d);
     }

void DeleteCursusMenu(void)
     {
	Cursus b;
	while (1)
	  {
	     b=SelectCursusMenu("Welke cursus deleten ?");
	     if (b) DeleteCursus(b);
	     else break;
	  }
     }

void EditCursusMenu(void)
     {
	Cursus b;
	while (1)
	  {
	     b=SelectCursusMenu("Welk cursus editeren ?");
	     if (b) EditCursus(b);
	     else break;
	  }
     }

void FindCursusMenu(void)
{SelectCursusMenu(NULL);}
