#include <strings.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <assert.h>
#include <errno.h>

#include "Book.h"
#include "BookActionServer.h"
#include "MemberActionServer.h"
#include "UitleningenActionServer.h"
#include "CursussenActionServer.h"
#include "Messages.h"
#include "Member.h"
#include "list.h"
#include "misc.h"
#include "intersection.h"


int StopServer=0;
int SocketDescriptor=0;
int SocketClient=0;
int KathySocket=0;
FILE *FakeSocketClient=NULL;
extern char *CurrentReceived;
char *CurrentReceivedOrig=NULL;
int readnet(void* watte,int size)
     {
	/* dat is hier nen toffen veraderlijken... */
	KathySocket=5;
	assert(watte && size);
	if (FakeSocketClient)
	  {
	     long s=fread(watte,1, size,FakeSocketClient);
	     return s;
	  }
	else 
	  {
	     int gelezen=0; /*,i;*/
	     while(gelezen!=size)
	       gelezen+=read(SocketClient,&((char*)watte)[gelezen],size-gelezen);
/*	     printf("%d: ",size);
	     for(i=0;i<size;i++)
	       {
		  printf("%d ",((char*)watte)[i]); fflush(stdout);
	       }
	     printf("\n"); fflush(stdout);
*/	     return size;
	  }
     }

int writenet(void* watte, int size)
     {
	if (FakeSocketClient) return size;
	return write(SocketClient,watte,size);
     }

int fileexists(const char*n)
     {
	FILE*f;
	f=fopen(n,"rb");
	if (!f) return 0;
	fclose(f);
	return 1;
     }

void DelRename(const char* old, const char*n)
     {
	if (fileexists(old)) remove(n);
	rename(old,n);
     }

void TryLoad(int(*loadfunc)(const char*),char*name)
     {
	char s[80];
	sprintf(s,"%s.dat",name);
	if (fileexists(s))
	  {
	     if (!loadfunc(s))
	       {
		  printf("IgServer: Couldn't load %s\n",name);
		  exit(1);
	       };
	  }
	else 
	  {
	     sprintf(s,"%s.old",name);
	     if (fileexists(s))
	       {
		  printf("IgServer: loading old %s\n",name);
		  if (!LoadMembers(s))
		    {
		       printf("IgServer: Couldn't load old %s !\n",name);
		       exit(1);
		    };
	       };
	  }
     }

void LoadAll(void)
     {
	printf("IgServer: Loading data files\n");
	TryLoad(LoadMembers,"Members");
	TryLoad(LoadBooks,"Books");
	TryLoad(LoadUitleningen,"Uitleningen");
	TryLoad(LoadCursussen,"Cursussen");
     }

void DoSave(int(*savefunc)(char*),char * str)
     {
	char s[80],t[80];
	sprintf(s,"%s.tmp",str);
	savefunc(s);
	sprintf(s,"%s.old",str);
	DelRename(t,s);
	sprintf(t,"%s.dat",str);
	DelRename(s,t);
     }

void SaveAll(void)
{
   /* eerst alles saven */
   printf("IgServer: Saving all data files\n");
   /* het saven is geen granulaire operatie. Daarom moet alles lukken
    * of alles failen !!! */
   if (SaveMembers("Members.tmp") &&
       SaveBooks("Books.tmp") &&
       SaveUitleningen("Uitleningen.tmp") &&
       SaveCursussen("Cursussen.tmp"))
     {
	DelRename("Members.dat","Members.old");
	DelRename("Books.dat","Books.old");
	DelRename("Uitleningen.dat","Uitleningen.old");
	DelRename("Cursussen.dat","Cursussen.old");
	
	DelRename("Members.tmp","Members.dat");
	DelRename("Books.tmp","Books.dat");
	DelRename("Uitleningen.tmp","Uitleningen.dat");
	DelRename("Cursussen.tmp","Cursussen.dat");
	
	DelRename("Messages.log","Messages.old");
     }
   else return;
}

char* ReadRequest(void)
{
   int i;
   unsigned int size;
   if (sizeof(size)!=readnet(&size,sizeof(size))) return NULL;
   size=ntohl(size);
   if (CurrentReceivedOrig) zfree(CurrentReceivedOrig);
   CurrentReceivedOrig=CurrentReceived=zmalloc(size);
   if (size!=readnet(CurrentReceived,size)) 
     {
	printf("IgServer: An error occured reading from socket...\n");
	return NULL;
     }
   /* het bericht saven indien niet gefaked*/
   if (!FakeSocketClient)
     {
 	FILE *messfile=fopen("Messages.log","ab");
	assert(messfile);
	fwrite(&size,1,sizeof(size),messfile);
	for(i=0;i<size;i++) fputc(CurrentReceived[i],messfile);
	fclose(messfile);
     }
   return CurrentReceived;
}

extern List AnswerList;
void FlushMessage(void)
     {
	DoFlushMessage();
     }

void STOPSERVER(char* message)
     {
	StopServer=1;
	SendChar(STOPSERVER_RP_OK);
	FlushMessage();
     }

typedef void (*ProcessMessageFunc)();
static ProcessMessageFunc Routines[16]=
     {
	ADDMEMBER,    /* [-- ok --] */
	STOPSERVER,   /* [-- ok --] */
	ADDBOOK,      /* [-- ok --] */
	FINDBOOKS,    /* [-- ok --] */ 	
	UPDATEBOOK,   /* [-- ok --] */
	DELETEBOOK,   /* [-- ok --] */
	UPDATEMEMBER, /* [-- ok --] */
	DELETEMEMBER, /* [-- ok --] */
	FINDMEMBERS,  /* [-- ok --] */
	LEENUIT,      /* [-- ok --] */
	FINDUITLENINGEN,    /* [-- ok --] */
	DELETEUITLENING,    /* [-- ok --] */
	ADDCURSUS,          /* [-- ok --] */
	DELETECURSUS,        /* [-- ok --] */
	UPDATECURSUS,        /* [-- ok --] */ 
	FINDCURSUSSEN        /* [-- ok --] */
     };

int RunNetwerk(void)
     {
	short sPort = 65503;
	struct sockaddr_in addrServer,addrClient;
	SocketDescriptor = socket(AF_INET, SOCK_STREAM, 0);
	if (SocketDescriptor==-1)
	  {
	     printf("IgServer: could not create socket\n");
	     exit(1);
	  }
	bzero(&addrServer,sizeof(addrServer));
	addrServer.sin_family = AF_INET;
	addrServer.sin_addr.s_addr = htonl(INADDR_ANY);
	addrServer.sin_port = htons(sPort);
	if (bind(SocketDescriptor, (struct sockaddr*)&addrServer, sizeof(addrServer))==-1)
	  {
	     printf("IgServer: Can't bind socket, errno=%d\n",errno);
	     exit(1);
	  };
	if (listen(SocketDescriptor, 100)==-1)
	  {
	     printf("IgServer: Can't listen to socket, errno=%d\n",errno);
	     exit(1);
	  };
	StopServer=0;
	self: if (getuid()) goto self;
	while (!StopServer)
	  {
	     char* message;
	     int iClientSize=sizeof(addrClient);
	     SocketClient = accept(SocketDescriptor, (struct sockaddr*)&addrClient, &iClientSize);
	     if (NULL==(message=ReadRequest())) StopServer=1;
	     else Routines[(int)RecvChar()]();
	     assert(l_empty(AnswerList));
	     close(SocketClient);
	  }
	close(SocketDescriptor);
	shutdown(SocketDescriptor,2);
	SocketDescriptor=0;
	return 1;
     }

int RunRecover(const char* name)
     {
	StopServer=0;
	FakeSocketClient=fopen(name,"rb");
	if (!FakeSocketClient) 
	  {
	     printf("IgServer: Can't open recover file\n");
	     exit(1);
	  }
	while (!feof(FakeSocketClient))
	  {
	     char* message;
	     if (NULL==(message=ReadRequest())) StopServer=1;
	     else Routines[(int)RecvChar()]();
	  }
	fclose(FakeSocketClient);
	return 1;
     }

void stop(int signum)
     {
	StopServer=1;
	signal(signum,stop);
     }

void InitServer(void)
     {
	/* zet een paar signal handlerkes */
	signal(SIGINT,stop);
	signal(SIGHUP,stop);
	signal(SIGQUIT,stop);
	InitIntersection();
     }

int main(int argc,char* argv[])
     {
	/* alles initialiseren */
	printf("IgServer: Initializing\n");
	InitServer();
	InitMemberServer();
	InitBookServer();
	InitUitleningenServer();
	InitCursussenServer();
	LoadAll();
	if (fileexists("Messages.log")) RunRecover("Messages.log");
	else RunNetwerk();
	SaveAll();
	printf("Server heeft zonet de geest gegeven...\n");
	return 0;
     }

