#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "misc.h"
#include "Uitlening.h"
#include "Member.h"
#include "Book.h"
#include "BookActionServer.h"
#include "MemberActionServer.h"
#include "UitleningenActionServer.h"
#include "list.h"
#include "Messages.h"
#include "intersection.h"
#include "Table.h"
#include "Record.h"

Table UitleningenTable=NULL;

void InitUitleningenServer(void)
     {UitleningenTable=AllocateTable(UITLENING_FIELD_COUNT);}
   
void LEENUIT(char*Message)
     {
	time_t tt;
	char* warranty;
	Member m;
	Book b;
	Uitlening u;
	m=(Member)RecvRecord(MEMBER_FIELD_COUNT);
	b=(Book)RecvRecord(BOOK_FIELD_COUNT);
	warranty=RecvString();
	/* vind het keynummer van beide */
	u=(Uitlening)AllocateRecord(UITLENING_FIELD_COUNT);
	u->BookKey=strdup(b->Key);
	u->Warranty=warranty;
	u->MemberKey=strdup(m->Key);
	u->Key=AllocateEmptyStr();
	tt=time(NULL);
	u->Date=strdup(ctime(&tt));
	u->Date[strlen(u->Date)-1]=0;
	AddRecord(UitleningenTable,(Record)u);
	/* antwoord sturen */
	FreeRecord((Record)b,BOOK_FIELD_COUNT);
	FreeRecord((Record)m,MEMBER_FIELD_COUNT);
	SendChar(LEENUIT_RP_OK);
	FlushMessage();
	printf("UitleningenServer: Added uitlening '%s <-> %s'\n",u->MemberKey,u->BookKey);
     }

void* SaveSingleUitlening(void* who, void* common)
     {
	FILE * F=(FILE*)common;
	int size;
	char*r;
	Uitlening u=(Uitlening)who;
	assert(u);
	assert(F);
	size=RecordFormattedSize((Record)u,UITLENING_FIELD_COUNT);
	r=(char*)zmalloc(size);
	FormatRecord(r,(Record)u,UITLENING_FIELD_COUNT);
	if ((1!=fwrite(&size,sizeof(size),1,F)) ||
	    (size!=fwrite(r,1,size,F)))
	  {
	     printf("UitleningenServer: Couldn't write uitlening outputfile\n");
	     exit(1);
	  };
	free(r);
	return who;
     }

int SaveUitleningen(const char * name)
     {
	FILE* o;
	o=fopen(name,"wb");
	if (!o) return 0;
	TableForeach(UitleningenTable,SaveSingleUitlening,o);
	fclose(o);
	printf("UitleningenServer: Saved uitleningen\n");
	return 1;
     }

int LoadUitleningen(const char*name)
     {
	int count;
	Uitlening b;
	char*r;
	FILE*i;
	i=fopen(name,"rb");
	if(!i)  return 0;
	count=0;
	while(!feof(i))
	  {
	     int s;
	     if (fread(&s,sizeof(int),1,i)!=1) break;
	     r=zmalloc(s);
	     if (fread(r,s,1,i)!=1) return 0;
	     RawAddRecord(UitleningenTable,b=UnFormatRecord(r,UITLENING_FIELD_COUNT));
	       {
		  int nr=atoi(b->Key);
		  if (nr>UitleningenTable->lastkeynumber) UitleningenTable->lastkeynumber=nr;
	       }
	     free(r);
	     count++;
	  }
	fclose(i);
	printf("UitleningenServer: Loaded %d uitleningen\n",count);
	return 1;
     }

void DELETEUITLENING(char*Message)
     {
	Uitlening u=(Uitlening)RecvRecord(UITLENING_FIELD_COUNT);
	DeleteRecord(UitleningenTable,(Record)u);
	SendChar(DELETEUITLENING_RP_OK);
	printf("UitleningenServer: Deleted uitlening '%s'\n",u->Key);
	FlushMessage();
	FreeRecord((Record)u,UITLENING_FIELD_COUNT);
     }

typedef struct svlamanswer
     {
	int size;
	char *MemberKey;
     } *vlamanswer;

void* CreateFindUitleningenAnswer(Uitlening u, vlamanswer a)
     {
	if (strcmp(u->MemberKey,a->MemberKey)==0)
	  {
	     Book b;
	     struct sUitleningSearchResult us;
	     a->size++;
	     b=(Book)Key2Record(BookTable,u->BookKey);
	     SetRecord((Record)&us.u,(Record)u,UITLENING_FIELD_COUNT);
	     SetRecord((Record)&us.B,(Record)b,BOOK_FIELD_COUNT);
	     SendRecord((Record)&us,BOOK_FIELD_COUNT+UITLENING_FIELD_COUNT);
	  }
	return u;
     }

void FINDUITLENINGEN(char*Message)
     {
	Member m;
	struct svlamanswer ta;
	m=(Member)RecvRecord(MEMBER_FIELD_COUNT);
	SendChar(FINDUITLENINGEN_RP_OK);
	SendChunck(&ta.size,4,dontfree);
	ta.size=0;
	ta.MemberKey=m->Key;
	l_foreach(UitleningenTable->l,CreateFindUitleningenAnswer,&ta);
	FreeRecord((Record)m,MEMBER_FIELD_COUNT);
	ta.size=htonl(ta.size);
	FlushMessage();
     }
