#include "myio.h"
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ncurses/ncurses.h>
#include <assert.h>
#include "misc.h"

#define KEUZE_ID 0
#define INPUT_ID 1
#define STATIC_ID 2
#define LISTBOX_ID 3

#define DD_NEXT 0
#define DD_PREV 1
#define DD_OK 2
#define DD_CANCEL 3

static int getcharacter(void);

struct DialogData {char id;};
struct listboxstruct  
     {
	struct DialogData d;  
	int x,y,dx,dy;   /* de window */
	char***contents;   /* de array van items */
	int breedte;    /* aantal fields in een item */
	int hoogte;     /* aantal items */
	char**header;   /* header */ 
	int current;    /* huidige selectie */
	int drawstart;  /* aantal lijnen overslaan bij listbox */
	int startx;     /* aantal karakters overslaan bij listboxline */
	void(*currentchanged)(void*record,void*common);   /* de callback als de huidige selectie wijzigt */
	void*commonforcc;
	void (*key)(int c,void*record,void*common);   /* de callback voor nen onbekenden toets */
	void*commonforkey;
     };

static void DrawDialogData(struct DialogData*dd);

static void DrawKeuze(struct DialogData*);
static int DoKeuze(struct DialogData*);
void FreeKeuze(keuze);

static void DrawInput(struct DialogData*);
static int DoInput(struct DialogData*);
void FreeInput(input);

static void DrawStaticText(struct DialogData*);
static int DoStaticText(struct DialogData*);
void FreeStaticText(statictext);

static void DrawListBox(struct DialogData*);
static int DoListBox(struct DialogData*);
void FreeListBox(listbox);

typedef void (*FreeViewFunc)(struct DialogData*);

struct sViewInfo
     {
	void (*DrawView)(struct DialogData*);
	int (*DoView)(struct DialogData*);
	FreeViewFunc FreeView;
     };

struct statictextstruct
     {
	struct DialogData d;
	int x,y;
	char* str;
     };

struct sViewInfo ViewInfo[4]=
     {
	  {DrawKeuze,DoKeuze,(FreeViewFunc)FreeKeuze},
	  {DrawInput,DoInput,(FreeViewFunc)FreeInput},
	  {DrawStaticText,DoStaticText,(FreeViewFunc)FreeStaticText},
	  {DrawListBox,DoListBox,(FreeViewFunc)FreeListBox}
     };


struct inputstruct
     {
	struct DialogData d;
	int x,y,aantal,curpos;
	char**str;
     };

struct keuzestruct 
     {
	struct DialogData d;
	int x,y; /* de positie op 't schermpje */
	int Current;
	int aantal;
	char **Keuzes;
     };

struct dialogstruct 
     {
	int aantal;
	int current;
	struct DialogData **Entrys;
     };


void SetListBoxContents(listbox lb,char***contents, int hoogte)
     {
	assert(lb);
	lb->contents=contents;
	lb->hoogte=hoogte;
	if (lb->current>=hoogte) lb->current=hoogte-1;
	if (lb->current<0) lb->current=0;
     }

int GetListBoxCurrent(listbox lb)
     {
	assert(lb);
	return lb->current;
     }

listbox MakeListBox(int x, int y, int dx, int dy,
		    char***contents, int breedte, int hoogte, char**Header)
     {
	listbox lb;
	lb=(listbox)zmalloc(sizeof(struct listboxstruct));
	lb->d.id=LISTBOX_ID;
	lb->x=x;
	lb->y=y;
	lb->dx=dx;
	lb->dy=dy;
	lb->contents=contents;
	lb->breedte=breedte;
	lb->hoogte=hoogte;
	lb->current=lb->drawstart=lb->startx=0;
	lb->header=Header;
	lb->key=lb->commonforkey=
	lb->currentchanged=lb->commonforcc=NULL;
	return lb;
     }

void SetListBoxCurrentChangedCallback(listbox lb,void(*callback)(void*record,void*common),void*common)
     {
	assert(lb);
	lb->currentchanged=callback;
	lb->commonforcc=common;
     }

void SetListBoxKeyCallback(listbox lb,void(*callback)(int c,void*record,void*common),void*common)
     {
	assert(lb);
	lb->key=callback;
	lb->commonforkey=common;
     }

statictext MakeStatic(int x, int y, char* str)
     {
	statictext s;
	s=(statictext)zmalloc(sizeof(struct statictextstruct));
	s->d.id=STATIC_ID;
	s->x=x;
	s->y=y;
	s->str=str;
	return s;
     }


void FreeStaticText(statictext s)
     {
	assert(s);
	free(s);
     };

void FreeListBox(listbox l)
     {
	assert(l);
	free(l);
     };

void FreeKeuze(keuze k)
{
   assert(k);
   assert(k->Keuzes);
   free(((keuze)k)->Keuzes);
   free(k);
}

void FreeInput(input p)
{
   assert(p);
   free(p);
}


int GetKeuzeCurrent(keuze k)
{
   assert(k);
   return k->Current;
}

input MakeInput(int x, int y, int aantal, char** s)
{
   input i;
   i=(input)zmalloc(sizeof(struct inputstruct));
   i->d.id=INPUT_ID;
   i->x=x;
   i->y=y;
   i->aantal=aantal;
   i->curpos=0;
   i->str=s;
   return i;
}

void SetInputString(input i, char** str)
     {
	/* **str is de referent van de string */
	/* setinputstring weizigt de referent */
	assert(i);
	i->curpos=0;
	i->str=str;
     }

void PasteInputString(input i, char* str)
     {
	assert(i);
	i->curpos=0;
	zfree(*(i->str));
	*(i->str)=strdup(str);
	i->curpos=0;
	DrawInput((struct DialogData*)i);
     }

keuze MakeKeuze(int pos, int x, int y, int aantal,...)
{
   keuze a;
   int i=0;
   va_list ap;
   a=(keuze)zmalloc(sizeof(struct keuzestruct));
   a->x=x;
   a->y=y;
   a->d.id=KEUZE_ID;
   a->Current=pos;
   a->aantal=aantal;
   a->Keuzes=(char**)zmalloc(sizeof(char*)*aantal);
   va_start(ap,aantal);
   while(aantal--)
     a->Keuzes[i++]=va_arg(ap,char*);
   va_end(ap);
   return a;
}

void FreeDialog(dialog d)
{
   assert(d);
   free(d);
}

void FreeDialogAndContents(dialog d)
     {
	int c;
	assert(d);
	c=d->aantal;
	while(--c>=0) ViewInfo[(int)d->Entrys[c]->id].FreeView(d->Entrys[c]);
	free(d);
     }

dialog MakeDialog(int aantal,char* Title,...)
     {
	int i=0;
	dialog d;
	va_list ap;
	d=(dialog)zmalloc(sizeof(struct dialogstruct));
	d->Entrys=(struct DialogData**)zmalloc(sizeof(struct DialogData*)*(aantal+1));
	d->aantal=aantal+1;
	d->current=0;
	d->Entrys[i++]=(struct DialogData*)MakeStatic(0,0,Title);
	va_start(ap,Title);
	while(aantal--)
	  d->Entrys[i++]=va_arg(ap,struct DialogData*);
	va_end(ap);
	return d;
     }

void SetDialogCurrent(dialog d, int i)
     {
	assert(d);
	d->current=i+1;
     }

void DrawDialogData(struct DialogData* dd)
     {
	assert(dd);
	ViewInfo[(int)dd->id].DrawView(dd);
     }

int RunDialogData(struct DialogData* dd)
     {
	assert(dd);
	assert(dd->id>=0 && dd->id<=3);
	return ViewInfo[(int)dd->id].DoView(dd);
     }

void RedrawDialog(dialog d)
     {
	int i=0,aantal;
	assert(d);
	aantal=d->aantal;
	clear();
	while(i<aantal)
	  DrawDialogData(d->Entrys[i++]);
	refresh();
     }

int RunDialog(dialog d)
     {
	int aantal;
	assert(d);
	/* ze eerst allemaal eens op het scherm zetten */
	aantal=d->aantal;
	assert(d->current<aantal && d->current>=0);
	RedrawDialog(d);
	/* en dan nu 1 voor 1 runnen */
	while(1)
	  {
	     int response;
	     response=RunDialogData(d->Entrys[d->current]);
	     switch(response)
	       {
		case DD_NEXT : 
		  d->current++;
		  if (d->current>=aantal) d->current=0;
		  break;
		case DD_PREV : 
		  d->current--;
		  if (d->current<0) d->current=aantal-1;
		  break;
		case DD_OK : return DIALOG_OK;
		case DD_CANCEL : return DIALOG_CANCEL;
		default: assert(0);
	       }
	  }
     }

/* something completely different */
static int getcharacter()
     {
	int y;
	while ((y=getch())==EOF);
	fflush(stdin);
	return y;
     }

void DrawInput(struct DialogData*dd)
{
   input p=(input)dd;
   int c;
   char*line;
   int t;
   assert(p);
   assert(p->str);
   line=*(p->str);
   t=p->aantal;
   attrset(A_BOLD);
   setsyx(p->y,p->x);
   c=p->curpos;
   if (line)
     {
	while(c+1>t) {line++; c--;}
	while(*line && t)
	  {
	     echochar(*line);
	     t--;
	     line++;
	  }
     }
   while(t) {echochar('_'); t--;}
   setsyx(p->y,p->x+c);
   refresh();
}

int DoInput(struct DialogData *dd)
{
   input p=(input)dd;
   int c, lastpos=0,bufsize=5;
   char*line;
   assert(p->str);
   line=*(p->str);
   curs_set(1);
   if (!line)  
     {
	*(p->str)=line=(char*)zmalloc(bufsize);
	*line=0;
     }
   else 
     {
	bufsize=strlen(line)+1;
	lastpos=bufsize-1;
     }
   DrawInput(dd);
   while (1)
     {
	c=getcharacter();
     	if (c>=' ' && c<='~')
	  {
	     assert(lastpos<bufsize);                       
	     if (lastpos+1==bufsize)
	       *(p->str)=line=(char*)zrealloc(line,bufsize+=5);
	     assert(line);                                   
	     memmove(&line[p->curpos+1],&line[p->curpos],lastpos-p->curpos+1);
	     line[p->curpos++]=c;
	     lastpos++;
	     DrawInput(dd);
	     continue;
	  }
	switch (c)
	  {
	   case 27:
	   case KEY_CANCEL: return DD_CANCEL;
	   case 10 : /* een geweldige enter */
	     return DD_OK;
	   case '\t' : /* den TAB */
	     return DD_NEXT;
	   case KEY_LEFT : /* pijltje links */
	     if (p->curpos) 
	       {
		  --(p->curpos);
		  DrawInput(dd);
	       }
	     continue;
	   case KEY_RIGHT : /* pijltje rechts */
	     if (p->curpos<lastpos) 
	       {
		  ++(p->curpos);
		  DrawInput(dd);
	       }
	     continue;
	   case KEY_HOME : /* home */
	     if (p->curpos)
	       {
		  assert(line);
		  p->curpos=0;
		  DrawInput(dd);
	       }
	     continue;
	   case KEY_END : /* end */
	     if (p->curpos!=lastpos)
	       {
		  assert(line);
		  p->curpos=lastpos;
		  DrawInput(dd);
	       }
	     continue;
	   case KEY_DC : /* del tsjoep */
	     if (p->curpos<lastpos)
	       {
		  assert(line);
		  memmove(&line[p->curpos],&line[p->curpos+1],lastpos-p->curpos); 
		  lastpos--;
		  DrawInput(dd);
	       }
	     continue;
	   case 263:
	   case 127:  /* empirisch vastgelegd dat dit den backspace is .. */
	     if (!p->curpos) continue;                             
	     assert(line);
	     memmove(&line[p->curpos-1],&line[p->curpos],lastpos-p->curpos+1);
	     lastpos--;
	     (p->curpos)--;
	     DrawInput(dd);
	     continue;
/*	   default:
	     printf("%d\n%d\n",c,c); fflush(stdout);*/
	  }
     }
}

void DrawStaticText(struct DialogData* dd)
{
   statictext s=(statictext)dd;
   char*line;
   assert(s);
   assert(s->str);
   line=s->str;
   attrset(A_NORMAL);
   setsyx(s->y,s->x);
   while(*line)
     {
	echochar(*line);
	line++;
     }
}

int DoStaticText(struct DialogData* dd)
     {
	return DD_NEXT;
     }

void InitMyIo(void)
  {
     initscr();
     start_color();
     cbreak(); 
     noecho();   
     keypad(stdscr,TRUE);
     notimeout(stdscr,TRUE);
  }

void DoneMyIo(void)
  {
     endwin();
  }

static keuzex=0;
static keuzey=0;
static void keuzeline(int selected, char* tekst)
     {
	int c;
	c=(selected ? A_BOLD : A_NORMAL);
	if (selected) echochar('*'|c);
	else echochar(' '|c);
	while(*tekst) {echochar((*tekst)|c); tekst++;}
     }

void DrawKeuze(struct DialogData* d)
{
   keuze k=(keuze)d;
   int temp,aantal;
   assert(d->id==KEUZE_ID);
   aantal=k->aantal;
   attrset(A_NORMAL);
   for(temp=0;temp<aantal;temp++) 
     {
	setsyx(k->y+temp,k->x);
	keuzeline(k->Current==temp,k->Keuzes[temp]);
     }
   wrefresh(stdscr);
}

static void listboxline(int selected,char** temp, char** tekst, int breedte, int dy, int starty)
     {
	int c,curfield,charsdrawn;
	char*s,*t;
	c=selected ? A_BOLD : A_NORMAL;
	curfield=0;
	charsdrawn=0;
	if (tekst)
	  while(curfield<breedte)
	  {
	     t=temp[curfield];
	     s=tekst[curfield];
	     assert(t && s);
	     while(*t)
	       {
		  if (*s) 
		    {
		       if (charsdrawn>=starty) 
			 {
			    echochar(*s|c);
			 }
		       s++;
		    }
		  else if (charsdrawn>=starty) echochar(' '|c);
		  charsdrawn++;
		  if (charsdrawn>=starty+dy) break;
		  t++;
	       }
	     curfield++;
	     if (charsdrawn>=starty+dy) break;
	     if (charsdrawn>=starty) echochar(' ');
	     charsdrawn++;
	     if (charsdrawn>=starty+dy) break;
	  }
	else charsdrawn=starty;
	while(charsdrawn++<starty+dy) echochar(' '|c);
     }

void DrawListBox(struct DialogData* d)
     {
	listbox lb=(listbox)d;
	int temp;
	assert(d->id==LISTBOX_ID);
	attrset(A_NORMAL);
	if (!lb->contents) return;
	setsyx(lb->y,lb->x);
	listboxline(1,lb->header,lb->header,lb->breedte,lb->dx,lb->startx);
	if (lb->current-lb->drawstart>=lb->dy-1) lb->drawstart=lb->current-lb->dy+2;
	if (lb->current<lb->drawstart) lb->drawstart=lb->current;
	if (lb->drawstart<0) lb->drawstart=0;
	for(temp=0;temp<lb->dy-1;temp++) 
	  {
             setsyx(lb->y+temp+1,lb->x);
	     listboxline(lb->current==temp+lb->drawstart,
			 lb->header,
			 lb->drawstart+temp<lb->hoogte ? lb->contents[temp+lb->drawstart] : NULL,
			 lb->breedte,
			 lb->dx,
			 lb->startx);
	  }
	wrefresh(stdscr);
     }

int DoListBox(struct DialogData*dd)
     {
	int c=0,b=0,prevcur;
	listbox lb=(listbox)dd;
	assert(lb);
	prevcur=-1;
	curs_set(0);
	do
	  {
	     DrawListBox(dd);
	     wrefresh(stdscr);
	     b=0;
	     if ((c=getcharacter())==10) return DD_OK;
	     if (c=='\t') return DD_NEXT;
	     if (c==KEY_CANCEL || c==27) return DD_CANCEL;
	     if (c==KEY_UP || c=='k') 
	       {
		  lb->current--;
		  b=1;
	       }
	     if (c==KEY_DOWN || c=='j') 
	       {
		  lb->current++;
		  b=1;
	       }
	     if (c==KEY_LEFT)
	       {
		  b=1;  
		  if (--(lb->startx)<0) lb->startx=0;
	       }
	     if (c==KEY_RIGHT) 
	       {
		  b=1;
		  lb->startx++;
	       }
	     if (c==KEY_HOME)
	       {
		  b=1;
		  if (lb->startx) lb->startx=0;
		  else lb->current=0;
	       }
	     if (c==KEY_END) 
	       {
		  lb->startx+=lb->dx;
		  b=1;
	       }
	     if (c==KEY_NPAGE) 
	       {
		  lb->current+=lb->dy-2;
		  b=1;
	       }
	     if (c==KEY_PPAGE) 
	       {
		  lb->current-=lb->dy-2;
		  b=1;
	       }
	     if (lb->current<0) lb->current=lb->hoogte-1;
	     if (lb->current>=lb->hoogte) lb->current=0;
	     if (prevcur!=lb->current)
	       {
		  prevcur=lb->current;
		  if (lb->currentchanged && lb->contents) lb->currentchanged(lb->contents[prevcur],lb->commonforcc);
	       }
	     if (!b && lb->key && lb->contents)
	       {
		  lb->key(c,lb->contents[prevcur],lb->commonforkey);
	       }
	  }
	while (1);
     }


int DoKeuze(struct DialogData*dd)
     {
	int c=0;
	keuze ddk=(keuze)dd;
	assert(ddk);
	curs_set(0);
	do
	  {
	     DrawKeuze(dd);
	     wrefresh(stdscr);
	     if ((c=getcharacter())==10) return DD_OK;
	     if (c=='\t') return DD_NEXT;
	     if (c==KEY_CANCEL || c==27) return DD_CANCEL;
	     if (c==KEY_UP || c=='k') ddk->Current--;
	     if (c==KEY_DOWN || c=='j') ddk->Current++;
	     if (ddk->Current<0) ddk->Current=ddk->aantal-1;
	     if (ddk->Current>=ddk->aantal) ddk->Current=0;
	  }
	while (1);
     }

int Keuze(int keuze, int escapeallowed, int aantal,...)
     {
	int c=0,temp;
	va_list ap;
	assert(keuze>=0 && keuze<=aantal);
	do
	  {
	     va_start(ap,aantal);
	     for(temp=0;temp<aantal;temp++) 
	       {
		  setsyx(keuzey+temp,keuzex);
		  keuzeline(keuze==temp,va_arg(ap,char*));
	       }
	     wrefresh(stdscr);
	     if ((c=getcharacter())==10) break;
	     if (c==KEY_CANCEL && escapeallowed)
	       {
		  va_end(ap);
		  return -1;
	       }
	     if (c==KEY_UP || c=='j') keuze--;
	     if (c==KEY_DOWN || c=='k') keuze++;
	     if (keuze<0) keuze=aantal-1;
	     if (keuze>=aantal) keuze=0;}
	while (1);
	va_start(ap,aantal);
	return keuze;
	va_end(ap);
	return -1;
     };

inputandstatic MakeInputAndStatic(int x,int y, int length, char* label,char**inputdata)
     {
	inputandstatic a;
	a.s=MakeStatic(x,y,label);
	a.i=MakeInput(strlen(label)+x,y,length,inputdata);
	return a;
     }

void FreeInputAndStatic(inputandstatic w)
     {
	FreeStaticText(w.s);
	FreeInput(w.i);
     }

void pressanykey()
     {
	getcharacter();
     }

void StatusLine(char* text)
/* de statusline heeft de onderste lijn opgeeist voor zichzelve */
     {
	int t=79;
	setsyx(23,1);
	attrset(A_NORMAL);
	while(*text && t)
	  {
	     echochar(*text);
	     t--;
	     text++;
	  }
	refresh();
	pressanykey();
	setsyx(23,1);
	clrtoeol();
	refresh();
     }

void FatalStatusLine(char* text)
     {
	StatusLine(text);
	DoneMyIo();
	exit(1);
     }
